from unittest import TestCase
from drmaa import *

class Misc(TestCase):

    def setUp(self):
        self.s = Session()

    def test_drmaa_get_contact(self):
        print self.s.contact

    def test_drmaa_version(self):
        print self.s.version

    def test_drmaa_get_DRM_system(self):
        print self.s.drmsInfo

    def test_drmaa_get_DRMAA_implementation(self):
        print self.s.drmaaImplementation

    def tearDown(self):
        self.s.exit()



# -----------------------------------------------------------
#  Copyright (C) 2009 StatPro Italia s.r.l. 
#                                                            
#  StatPro Italia                                            
#  Via G. B. Vico 4                                          
#  I-20123 Milano                                            
#  ITALY                                                     
#                                                            
#  phone: +39 02 96875 1                                     
#  fax:   +39 02 96875 605                                   
#                                                            
#  email: info@riskmap.net                                   
#                                                            
#  This program is distributed in the hope that it will be   
#  useful, but WITHOUT ANY WARRANTY; without even the        
#  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR   
#  PURPOSE. See the license for more details.                
# -----------------------------------------------------------

"""test module for the functional interface"""

from drmaa import *
import unittest

def setup():
    "initialize DRMAA library"
    Session.initialize()

def teardown():
    "finalize DRMAA session"
    Session.exit()

def test_allocate():
    "job template allocation"
    jt = Session.createJobTemplate()
    Session.deleteJobTemplate(jt)

class SubmitBase(unittest.TestCase):

    def setUp(self):
        self.jt = jt = Session.createJobTemplate()
        jt.args = ['2']
        jt.remoteCommand = 'sleep'
        self.jid = Session.runJob(jt)

    def tearDown(self):
        Session.deleteJobTemplate(self.jt)

class Submit(SubmitBase):

    def test_run_bulk(self):
        "run bulk job"
        jids = Session.runBulkJobs(self.jt, 1, 2, 1)

    def test_wait(self):
        "waiting for job completion"
        jinfo = Session.wait(self.jid)
        print jinfo

    def test_sync(self):
        "sync with a job"
        Session.synchronize(self.jid)
        

class JobTemplateTests(unittest.TestCase):
    def setUp(self):
        self.jt = Session.createJobTemplate()

    def test_scalar_attributes(self):
        for name, value in [("remoteCommand"           ,'pippo'),
                            ("jobSubmissionState"      ,'drmaa_active'),
                            ("workingDirectory"        ,'pippo'),
                            ("jobCategory"             ,'pippo'),
                            ("nativeSpecification"     ,'-shell yes'),
                            ("blockEmail"              , False),
                            ("startTime"               ,'00:01'),
                            ("jobName"                 ,'pippo'),
                            ("inputPath"               ,'/tmp/input'),
                            ("outputPath"              ,'/tmp/output'),
                            ("errorPath"               ,'/tmp/error'),
                            ("joinFiles"               ,True),]:
                            #("transferFiles"           ,'i')
                            #("deadlineTime"            ,'10'),
                            #("hardWallclockTimeLimit"  ,'10:00'),
                            #("softWallclockTimeLimit"  ,'00:00:10'),]:
                            #("hardRunDurationLimit"    ,'00:01:00'),
                            #("softRunDurationLimit"    ,'00:01:00')]:
            setattr(self.jt, name, value)
            assert getattr(self.jt, name) == value
            

    def test_vector_attributes(self):
        args = ['10', 'de', 'arglebargle']
        self.jt.args = args
        assert self.jt.args == args
        em = ['baz@quz.edu', 'foo@bar.com']
        self.jt.email = em
        assert self.jt.email == em

    def test_dict_attribute(self):
        from os import environ
        self.jt.environment = environ
        assert environ == self.jt.environment

    def test_attribute_names(self):
        assert len(self.jt.attributeNames) > 0
    
    def tearDown(self):
        Session.deleteJobTemplate(self.jt)



# -----------------------------------------------------------
#  Copyright (C) 2008 StatPro Italia s.r.l. 
#                                                            
#  StatPro Italia                                            
#  Via G. B. Vico 4                                          
#  I-20123 Milano                                            
#  ITALY                                                     
#                                                            
#  phone: +39 02 96875 1                                     
#  fax:   +39 02 96875 605                                   
#                                                            
#  This program is distributed in the hope that it will be   
#  useful, but WITHOUT ANY WARRANTY; without even the        
#  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR   
#  PURPOSE. See the license for more details.                
# -----------------------------------------------------------
#  
#  Author: Enrico Sirola <enrico.sirola@statpro.com> 

import ctypes as _ct

try:
    import namedtuple as _nt
except ImportError: # pre 2.6 behaviour
    import nt as _nt

import drmaa.const as _c
from drmaa.const import JobState, JobControlAction, JobSubmissionState
import drmaa.wrappers as _w
import drmaa.helpers as _h
from drmaa.errors import (AlreadyActiveSessionException,
                          AuthorizationException,
                          ConflictingAttributeValuesException,
                          DefaultContactStringException,
                          DeniedByDrmException,
                          DrmCommunicationException,
                          DrmsExitException,
                          DrmsInitException,
                          ExitTimeoutException,
                          HoldInconsistentStateException,
                          IllegalStateException,
                          InternalException,
                          InvalidAttributeFormatException,
                          InvalidContactStringException,
                          InvalidJobException,
                          InvalidJobTemplateException,
                          NoActiveSessionException,
                          NoDefaultContactStringSelectedException,
                          ReleaseInconsistentStateException,
                          ResumeInconsistentStateException,
                          SuspendInconsistentStateException,
                          TryLaterException,
                          UnsupportedAttributeException,
                          InvalidArgumentException,
                          InvalidAttributeValueException,
                          OutOfMemoryException,)

Version = _h.Version
JobInfo = _nt.namedtuple("JobInfo", 
                         """jobId hasExited hasSignal terminatedSignal 
                            hasCoreDump wasAborted resourceUsage""")
# FileTransferMode = _nt.namedtuple("FileTransferMode", 
#                                   """transferInputStream transferOutputStream 
#                                      transferErrorStream""")

class JobTemplate(object):

    HOME_DIRECTORY='$drmaa_hd_ph$'
    WORKING_DIRECTORY='$drmaa_wd_ph$'
    PARAMETRIC_INDEX='$drmaa_incr_ph$'
    # contains list of strings

    # this should probably return the JobTemplate attribute names, not the
    # C API drmaa attribute names
    @property
    def attributeNames(self):
        return list(_h.attribute_names_iterator())

    # scalar attributes
    remoteCommand           = _h.Attribute(_c.REMOTE_COMMAND       )
    jobSubmissionState      = _h.Attribute(_c.JS_STATE             )
    workingDirectory        = _h.Attribute(_c.WD                   )
    jobCategory             = _h.Attribute(_c.JOB_CATEGORY         )
    nativeSpecification     = _h.Attribute(_c.NATIVE_SPECIFICATION )
    blockEmail              = _h.Attribute(_c.BLOCK_EMAIL, type_converter=_h.BoolConverter)
    startTime               = _h.Attribute(_c.START_TIME           )
    jobName                 = _h.Attribute(_c.JOB_NAME             )
    inputPath               = _h.Attribute(_c.INPUT_PATH           )
    outputPath              = _h.Attribute(_c.OUTPUT_PATH          )
    errorPath               = _h.Attribute(_c.ERROR_PATH           )
    joinFiles               = _h.Attribute(_c.JOIN_FILES, type_converter=_h.BoolConverter)
    # the following is available on ge6.2 only if enabled via cluster configuration
    transferFiles           = _h.Attribute(_c.TRANSFER_FILES       )
    # the following are apparently not available on ge 6.2
    # it will raise if you try to access these attrs
    deadlineTime            = _h.Attribute(_c.DEADLINE_TIME        )
    # these will also probably need specific converters (should receive float values)
    hardWallclockTimeLimit  = _h.Attribute(_c.WCT_HLIMIT           )
    softWallclockTimeLimit  = _h.Attribute(_c.WCT_SLIMIT           )
    hardRunDurationLimit    = _h.Attribute(_c.DURATION_HLIMIT      )
    softRunDurationLimit    = _h.Attribute(_c.DURATION_SLIMIT      )

    # vector attributes
    email = _h.VectorAttribute(_c.V_EMAIL)
    args  = _h.VectorAttribute(_c.V_ARGV)
    
    # dict attributes
    environment = _h.DictAttribute(_c.V_ENV)

    _as_parameter_ = None

    def __init__(self):
        jt = _ct.pointer(_ct.POINTER(_w.drmaa_job_template_t)())
        _h.c(_w.drmaa_allocate_job_template, jt)
        self._jt = self._as_parameter_ = jt.contents

    def delete(self):
        _h.c(_w.drmaa_delete_job_template, self)

    def __enter__(self):
        return self

    def __exit__(self, *args):
        self.delete()
        return False

class Session(object):
    TIMEOUT_WAIT_FOREVER=-1
    TIMEOUT_NO_WAIT=0
    JOB_IDS_SESSION_ANY='DRMAA_JOB_IDS_SESSION_ANY'
    JOB_IDS_SESSION_ALL='DRMAA_JOB_IDS_SESSION_ALL'

    contact = _h.SessionStringAttribute(_w.drmaa_get_contact)
    drmsInfo = _h.SessionStringAttribute(_w.drmaa_get_DRM_system)
    drmaaImplementation = _h.SessionStringAttribute(_w.drmaa_get_DRMAA_implementation)
    version = _h.SessionVersionAttribute()

    def __init__(self, contactString=''):
        self.initialize(contactString)

    # no return value
    @staticmethod
    def initialize(contactString=''):
        _w.init(contactString)

    # no return value
    @staticmethod
    def exit():
        _w.exit()

    # returns JobTemplate instance
    @staticmethod
    def createJobTemplate():
        return JobTemplate()

    # takes JobTemplate instance, no return value
    @staticmethod
    def deleteJobTemplate(jobTemplate):
        jobTemplate.delete()

    # takes JobTemplate instance, returns string
    @staticmethod
    def runJob(jobTemplate):
        jid = _ct.create_string_buffer(128)
        _h.c(_w.drmaa_run_job, jid, _ct.sizeof(jid), jobTemplate)
        return jid.value

    # takes JobTemplate instance and num values, returns string list
    @staticmethod
    def runBulkJobs(jobTemplate, beginIndex, endIndex, step):
        return list(_h.run_bulk_job(jobTemplate, beginIndex, endIndex, step))

    # takes string and JobControlAction value, no return value
    @staticmethod
    def control(jobName, operation):
        _w.drmaa_control(jobName, _c.string_to_control_action(operation))

    # takes string list, num value and boolean, no return value
    @staticmethod
    def synchronize(jobList, timeout=-1, dispose=False):
        if dispose: d=1
        else: d=0
        _h.c(_w.drmaa_synchronize, _h.string_vector(jobList), timeout, d)

    # takes string and long, returns JobInfo instance
    @staticmethod
    def wait(jobName, timeout=-1):
        stat = _ct.c_int()
        rusage = _ct.pointer(_ct.POINTER(_w.drmaa_attr_values_t)())
        _h.c(_w.drmaa_wait, jobName, None, 0, _ct.byref(stat), 
             timeout, rusage)
        res_usage = _h.adapt_rusage(rusage)
        exited = _ct.c_int()
        _h.c(_w.drmaa_wifexited, _ct.byref(exited), stat)
        aborted = _ct.c_int()
        _h.c(_w.drmaa_wifaborted, _ct.byref(aborted), stat)
        signaled = _ct.c_int()
        _h.c(_w.drmaa_wifsignaled, _ct.byref(signaled), stat)
        coredumped = _ct.c_int()
        _h.c(_w.drmaa_wcoredump, _ct.byref(coredumped), stat)
        exit_status = _ct.c_int()
        _h.c(_w.drmaa_wexitstatus, _ct.byref(exit_status), stat)
        term_signal = _ct.create_string_buffer(_c.SIGNAL_BUFFER)
        _h.c(_w.drmaa_wtermsig, term_signal, _ct.sizeof(term_signal), stat)
        return JobInfo(jobName, bool(exited), bool(signaled), term_signal.value, 
                       bool(coredumped), bool(aborted), res_usage)

    # takes string, returns JobState instance
    @staticmethod
    def jobStatus(jobName):
        status = _ct.c_int()
        _h.c(_w.drmaa_job_ps, jobName, _ct.byref(status))
        return _c.status_to_string(status.value)

    def __enter__(self):
        return self

    def __exit__(self, *args):
        self.exit()
        return False



from __future__ import with_statement
from drmaa import *

def test_with_session():
    with Session() as s:
        print s.version
        print s.contact
        print s.drmsInfo
        print s.drmaaImplementation

def test_with_jt():
    s = Session()
    with s.createJobTemplate() as jt:
        jt.remoteCommand = 'sleep'
        jt.args = ['10']
        jid = s.runJob(jt)
        print s.wait(jid)
    s.exit()




#!/bin/python
import drmaa
import os
from sys import argv, exit
from time import sleep
from pprint import pprint

NBULKS = 1
JOB_CHUNK = 3

OPATH=":"+drmaa.JobTemplate.HOME_DIRECTORY+'/DRMAA_JOB_OUT'

def init_job_template(jt, path, args, as_bulk_job):
    jt.workingDirectory = drmaa.JobTemplate.HOME_DIRECTORY
    jt.environment = {'a': 'b', 'c':'d',}
    jt.remoteCommand = path
    jt.args = args
    jt.joinFiles=True
    #jt.startTime=datetime.now()
    #print "start time is: ", jt.startTime
    if as_bulk_job:
        jt.outputPath=OPATH+'.'+drmaa.JobTemplate.PARAMETRIC_INDEX
    else:
        jt.outputPath=OPATH
    return jt


def main():
    if len(argv) < 2:
        print "usage: example.py <path-to-job> <arguments>"
        exit(1)
    job_path=argv[1]
    s=drmaa.Session()
    # submit some bulk jobs
    jt=init_job_template(s.createJobTemplate(), job_path, argv[2:], True)
    all_jobids = []
    for i in range(NBULKS):
        all_jobids += s.runBulkJobs(jt, 1, JOB_CHUNK, 1)
        sleep(1)
    print "submitted bulk jobs with jobids:"
    pprint(all_jobids)
    # submit some sequential jobs
    s.deleteJobTemplate(jt)
    del jt
    jt=init_job_template(s.createJobTemplate(), job_path, argv[2:], False)
    for i in range(NBULKS):
        all_jobids.append(s.runJob(jt))
        sleep(1)
    s.synchronize(all_jobids,
                  drmaa.Session.TIMEOUT_WAIT_FOREVER,
                  False)
    print "synchronized with all jobs"
    for jid in all_jobids:
        print '-' * 76
        info=s.wait(jid, drmaa.Session.TIMEOUT_WAIT_FOREVER)
        print """\
id:                        %(jobId)s
exited:                    %(hasExited)s
signaled:                  %(hasSignal)s
with signal (id signaled): %(terminatedSignal)s
dumped core:               %(hasCoreDump)s
aborted:                   %(wasAborted)s
resource usage:

%(resourceUsage)s
""" % info._asdict()

if __name__=='__main__':
    main()



# -----------------------------------------------------------
#  Copyright (C) 2009 StatPro Italia s.r.l. 
#                                                            
#  StatPro Italia                                            
#  Via G. B. Vico 4                                          
#  I-20123 Milano                                            
#  ITALY                                                     
#                                                            
#  phone: +39 02 96875 1                                     
#  fax:   +39 02 96875 605                                   
#                                                            
#  email: info@riskmap.net                                   
#                                                            
#  This program is distributed in the hope that it will be   
#  useful, but WITHOUT ANY WARRANTY; without even the        
#  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR   
#  PURPOSE. See the license for more details.                
# -----------------------------------------------------------
#  
#  Author: Enrico Sirola <enrico.sirola@statpro.com> 

"""DRMAA C library function wrappers"""

from ctypes import * 
from ctypes.util import find_library
from drmaa.errors import error_check, error_buffer

libpath = find_library('drmaa')

if libpath is None:
    import os
    from os.path import join
    import sys
    if sys.platform.startswith('linux') \
           and 'SGE_ROOT' in os.environ \
           and 'LD_LIBRARY_PATH' in os.environ:
        # linux hack
        libpath = join([ x for x in os.environ['LD_LIBRARY_PATH'].split(':')
                         if x.startswith(os.environ['SGE_ROOT']) ][0], 'libdrmaa.so')
    else:
        raise RuntimeError('cannot find drmaa library, please set the libpath variable by hand into file %s' % __file__)

_lib = cdll.LoadLibrary(libpath)

STRING = c_char_p
size_t = c_ulong
ptrdiff_t = c_int

drmaa_init = _lib.drmaa_init
drmaa_init.restype = error_check
drmaa_init.argtypes = [STRING, STRING, size_t]
drmaa_exit = _lib.drmaa_exit
drmaa_exit.restype = error_check
drmaa_exit.argtypes = [STRING, size_t]

def init(contact=None):
    return _lib.drmaa_init(contact, error_buffer, sizeof(error_buffer))

_lib.drmaa_exit.argtypes = [c_char_p, c_size_t] 
_lib.drmaa_init.restype = error_check 
def exit():
    return _lib.drmaa_exit(error_buffer, sizeof(error_buffer))


# structures
class drmaa_job_template_s(Structure):
    pass
drmaa_job_template_t = drmaa_job_template_s
drmaa_job_template_s._fields_ = [
]
class drmaa_attr_names_s(Structure):
    pass
drmaa_attr_names_t = drmaa_attr_names_s
drmaa_attr_names_s._fields_ = [
]
class drmaa_attr_values_s(Structure):
    pass
drmaa_attr_values_t = drmaa_attr_values_s
drmaa_attr_values_s._fields_ = [
]
class drmaa_job_ids_s(Structure):
    pass
drmaa_job_ids_t = drmaa_job_ids_s
drmaa_job_ids_s._fields_ = [
]

drmaa_get_contact = _lib.drmaa_get_contact
drmaa_get_contact.restype = error_check
drmaa_get_contact.argtypes = [STRING, size_t, STRING, size_t]
drmaa_version = _lib.drmaa_version
drmaa_version.restype = error_check
drmaa_version.argtypes = [POINTER(c_uint), POINTER(c_uint), STRING, size_t]
drmaa_get_DRM_system = _lib.drmaa_get_DRM_system
drmaa_get_DRM_system.restype = error_check
drmaa_get_DRM_system.argtypes = [STRING, size_t, STRING, size_t]
drmaa_get_DRMAA_implementation = _lib.drmaa_get_DRMAA_implementation
drmaa_get_DRMAA_implementation.restype = error_check
drmaa_get_DRMAA_implementation.argtypes = [STRING, size_t, STRING, size_t]

drmaa_allocate_job_template = _lib.drmaa_allocate_job_template
drmaa_allocate_job_template.restype = error_check
drmaa_allocate_job_template.argtypes = [POINTER(POINTER(drmaa_job_template_t)), STRING, size_t]
drmaa_delete_job_template = _lib.drmaa_delete_job_template
drmaa_delete_job_template.restype = error_check
drmaa_delete_job_template.argtypes = [POINTER(drmaa_job_template_t), STRING, size_t]
drmaa_set_attribute = _lib.drmaa_set_attribute
drmaa_set_attribute.restype = error_check
drmaa_set_attribute.argtypes = [POINTER(drmaa_job_template_t), STRING, 
                                STRING, STRING, size_t]
drmaa_get_attribute = _lib.drmaa_get_attribute
drmaa_get_attribute.restype = error_check
drmaa_get_attribute.argtypes = [POINTER(drmaa_job_template_t), STRING, 
                                STRING, size_t, STRING, size_t]

drmaa_get_next_attr_name = _lib.drmaa_get_next_attr_name
drmaa_get_next_attr_name.restype = c_int
drmaa_get_next_attr_name.argtypes = [POINTER(drmaa_attr_names_t), STRING, size_t]
drmaa_get_next_attr_value = _lib.drmaa_get_next_attr_value
drmaa_get_next_attr_value.restype = c_int
drmaa_get_next_attr_value.argtypes = [POINTER(drmaa_attr_values_t), STRING, size_t]
drmaa_get_next_job_id = _lib.drmaa_get_next_job_id
drmaa_get_next_job_id.restype = error_check
drmaa_get_next_job_id.argtypes = [POINTER(drmaa_job_ids_t), STRING, size_t]
drmaa_release_attr_names = _lib.drmaa_release_attr_names
drmaa_release_attr_names.restype = None
drmaa_release_attr_names.argtypes = [POINTER(drmaa_attr_names_t)]
drmaa_release_attr_values = _lib.drmaa_release_attr_values
drmaa_release_attr_values.restype = None
drmaa_release_attr_values.argtypes = [POINTER(drmaa_attr_values_t)]
drmaa_release_job_ids = _lib.drmaa_release_job_ids
drmaa_release_job_ids.restype = None
drmaa_release_job_ids.argtypes = [POINTER(drmaa_job_ids_t)]

drmaa_set_vector_attribute = _lib.drmaa_set_vector_attribute
drmaa_set_vector_attribute.restype = error_check
drmaa_set_vector_attribute.argtypes = [POINTER(drmaa_job_template_t), STRING, 
                                       POINTER(STRING), STRING, size_t]
drmaa_get_vector_attribute = _lib.drmaa_get_vector_attribute
drmaa_get_vector_attribute.restype = error_check
drmaa_get_vector_attribute.argtypes = [POINTER(drmaa_job_template_t), STRING, 
                                       POINTER(POINTER(drmaa_attr_values_t)), STRING, size_t]
drmaa_get_attribute_names = _lib.drmaa_get_attribute_names
drmaa_get_attribute_names.restype = error_check
drmaa_get_attribute_names.argtypes = [POINTER(POINTER(drmaa_attr_names_t)), STRING, size_t]
drmaa_get_vector_attribute_names = _lib.drmaa_get_vector_attribute_names
drmaa_get_vector_attribute_names.restype = error_check
drmaa_get_vector_attribute_names.argtypes = [POINTER(POINTER(drmaa_attr_names_t)), STRING, size_t]

try:
    drmaa_get_num_attr_names = _lib.drmaa_get_num_attr_names
    drmaa_get_num_attr_names.restype = c_int
    drmaa_get_num_attr_names.argtypes = [POINTER(drmaa_attr_names_t), POINTER(c_int)]
    drmaa_get_num_attr_values = _lib.drmaa_get_num_attr_values
    drmaa_get_num_attr_values.restype = c_int
    drmaa_get_num_attr_values.argtypes = [POINTER(drmaa_attr_values_t), POINTER(c_int)]
except AttributeError: # the above are present from 1.0 onward only
    pass

drmaa_run_job = _lib.drmaa_run_job
drmaa_run_job.restype = error_check
drmaa_run_job.argtypes = [STRING, size_t, POINTER(drmaa_job_template_t), STRING, size_t]
drmaa_run_bulk_jobs = _lib.drmaa_run_bulk_jobs
drmaa_run_bulk_jobs.restype = error_check
drmaa_run_bulk_jobs.argtypes = [POINTER(POINTER(drmaa_job_ids_t)), 
                                POINTER(drmaa_job_template_t), 
                                c_int, c_int, c_int, STRING, size_t]
drmaa_control = _lib.drmaa_control
drmaa_control.restype = error_check
drmaa_control.argtypes = [STRING, c_int, STRING, size_t]
drmaa_synchronize = _lib.drmaa_synchronize
drmaa_synchronize.restype = error_check
drmaa_synchronize.argtypes = [POINTER(STRING), c_long, c_int, STRING, size_t]
drmaa_wait = _lib.drmaa_wait
drmaa_wait.restype = error_check
drmaa_wait.argtypes = [STRING, STRING, size_t, POINTER(c_int), c_long, 
                       POINTER(POINTER(drmaa_attr_values_t)), STRING, size_t]
drmaa_wifexited = _lib.drmaa_wifexited
drmaa_wifexited.restype = error_check
drmaa_wifexited.argtypes = [POINTER(c_int), c_int, STRING, size_t]
drmaa_wexitstatus = _lib.drmaa_wexitstatus
drmaa_wexitstatus.restype = error_check
drmaa_wexitstatus.argtypes = [POINTER(c_int), c_int, STRING, size_t]
drmaa_wifsignaled = _lib.drmaa_wifsignaled
drmaa_wifsignaled.restype = error_check
drmaa_wifsignaled.argtypes = [POINTER(c_int), c_int, STRING, size_t]
drmaa_wtermsig = _lib.drmaa_wtermsig
drmaa_wtermsig.restype = error_check
drmaa_wtermsig.argtypes = [STRING, size_t, c_int, STRING, size_t]
drmaa_wcoredump = _lib.drmaa_wcoredump
drmaa_wcoredump.restype = error_check
drmaa_wcoredump.argtypes = [POINTER(c_int), c_int, STRING, size_t]
drmaa_wifaborted = _lib.drmaa_wifaborted
drmaa_wifaborted.restype = error_check
drmaa_wifaborted.argtypes = [POINTER(c_int), c_int, STRING, size_t]
drmaa_job_ps = _lib.drmaa_job_ps
drmaa_job_ps.restype = error_check
drmaa_job_ps.argtypes = [STRING, POINTER(c_int), STRING, size_t]
drmaa_strerror = _lib.drmaa_strerror
drmaa_strerror.restype = STRING
drmaa_strerror.argtypes = [c_int]



# -----------------------------------------------------------
#  Copyright (C) 2009 StatPro Italia s.r.l. 
#                                                            
#  StatPro Italia                                            
#  Via G. B. Vico 4                                          
#  I-20123 Milano                                            
#  ITALY                                                     
#                                                            
#  phone: +39 02 96875 1                                     
#  fax:   +39 02 96875 605                                   
#                                                            
#  This program is distributed in the hope that it will be   
#  useful, but WITHOUT ANY WARRANTY; without even the        
#  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR   
#  PURPOSE. See the license for more details.                
# -----------------------------------------------------------
#  
#  Author: Enrico Sirola <enrico.sirola@statpro.com> 

from setuptools import setup, find_packages

setup(
    name="drmaa",
    version="0.4a1",
    packages=find_packages(),
    package_data={'': ['*.txt', '*.rst']},
    author="Enrico Sirola",
    author_email="enrico.sirola@gmail.com",
    description="a python DRMAA library",
    license="BSD",
    keywords="python grid hpc drmaa",
    url="http://drmaa-python.googlecode.com",
    download_url="http://code.google.com/p/drmaa-python/downloads/list",
    tests_require='nose',
    test_suite='nose.collector',
    classifiers="""\
Development Status :: 3 - Alpha
Intended Audience :: Developers
Operating System :: OS Independent
Intended Audience :: System Administrators
License :: OSI Approved :: BSD License
Programming Language :: Python :: 2
Topic :: Software Development :: Libraries :: Python Modules
Topic :: System :: Distributed Computing""".split('\n'),
)
    




# -----------------------------------------------------------
#  Copyright (C) 2009 StatPro Italia s.r.l. 
#                                                            
#  StatPro Italia                                            
#  Via G. B. Vico 4                                          
#  I-20123 Milano                                            
#  ITALY                                                     
#                                                            
#  phone: +39 02 96875 1                                     
#  fax:   +39 02 96875 605                                   
#                                                            
#  email: info@riskmap.net                                   
#                                                            
#  This program is distributed in the hope that it will be   
#  useful, but WITHOUT ANY WARRANTY; without even the        
#  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR   
#  PURPOSE. See the license for more details.                
# -----------------------------------------------------------
#  
#  Author: Enrico Sirola <enrico.sirola@statpro.com> 

"""internal helpers"""

import ctypes as _ct
from drmaa.wrappers import *
from drmaa.errors import error_buffer
import drmaa.const as const

try:
    import namedtuple as _nt
except ImportError: # pre 2.6 behaviour
    import nt as _nt

class BoolConverter(object):

    @staticmethod
    def to_drmaa(value):
        if value: return 'y'
        else: return 'n'
    @staticmethod
    def from_drmaa(value):
        if value == 'y':
            return True
        else:
            return False

class SessionStringAttribute(object):

    def __init__(self, drmaa_function):
        self._f = drmaa_function
    def __get__(self, *args):
        buf = _ct.create_string_buffer(1024)
        c(self._f, buf, _ct.sizeof(buf))
        return buf.value

Version = _nt.namedtuple("Version", "major minor")
Version.__str__ = lambda x: "%s.%s" % (x.major, x.minor)

class SessionVersionAttribute(object):

    def __get__(self, *args):
        major = _ct.c_uint(10)
        minor = _ct.c_uint(10)
        c(drmaa_version, _ct.byref(major), _ct.byref(minor))
        return Version(major.value, minor.value)

class Attribute(object):

    def __init__(self, name, type_converter=None):
        self.name = name
        self.converter = type_converter
    def __set__(self, instance, value):
        if self.converter:
            v = self.converter.to_drmaa(value)
        else:
            v = value
        c(drmaa_set_attribute, instance, self.name, v)
    def __get__(self, instance, _):
        attr_buffer = create_string_buffer(const.ATTR_BUFFER)
        c(drmaa_get_attribute, instance, self.name, 
          attr_buffer, sizeof(attr_buffer))
        if self.converter:
            return self.converter.from_drmaa(attr_buffer.value)
        else:
            return attr_buffer.value

class VectorAttribute(object):

    def __init__(self, name):
        self.name = name
    def __set__(self, instance, value):
        c(drmaa_set_vector_attribute, instance, self.name, string_vector(value))
    def __get__(self, instance, _):
        return list(vector_attribute_iterator(
                instance, self.name))

class DictAttribute(object):

    def __init__(self, name):
        self.name = name
    def __set__(self, instance, value):
        v = [ "%s=%s" % (k, v) for (k, v) in value.iteritems() ]
        c(drmaa_set_vector_attribute, instance, self.name, string_vector(v))
    def __get__(self, instance, _):
        return dict([ i.split('=') for i in list(vector_attribute_iterator(
                        instance, self.name)) ])


def attributes_iterator(attributes):
    try:
        buf = create_string_buffer(const.ATTR_BUFFER)
        while drmaa_get_next_attr_value(attributes, buf, sizeof(buf))\
                != const.NO_MORE_ELEMENTS:
            yield buf.value
    finally:
        drmaa_release_attr_values(attributes)

def adapt_rusage(rusage):
    "transform a rusage data structure into a dict"
    rv = dict()
    for attr in attributes_iterator(rusage.contents):
        k, v = attr.split('=')
        rv[k] = v
    return rv

def vector_attribute_iterator(jt, attr_name):
    avalues = pointer(POINTER(drmaa_attr_values_t)())
    c(drmaa_get_vector_attribute, jt, attr_name, avalues)
    return attributes_iterator(avalues.contents)

def attribute_names_iterator():
    attrn_p = pointer(POINTER(drmaa_attr_names_t)())
    c(drmaa_get_attribute_names, attrn_p)
    try: 
        name = create_string_buffer(128)
        while drmaa_get_next_attr_name(attrn_p.contents, name, 128)\
                != const.NO_MORE_ELEMENTS:
            yield name.value
    finally:
        drmaa_release_attr_names(attrn_p.contents)

def vector_attribute_names_iterator():
    attrn_p = pointer(POINTER(drmaa_attr_names_t)())
    c(drmaa_get_vector_attribute_names, attrn_p)
    try:
        name = create_string_buffer(128)
        while drmaa_get_next_attr_name(attrn_p.contents, name, 128)\
                != const.NO_MORE_ELEMENTS:
            yield name.value
    finally:
        drmaa_release_attr_names(attrn_p.contents)

def run_bulk_job(jt, start, end, incr=1):
    jids = pointer(POINTER(drmaa_job_ids_t)())
    try:
        c(drmaa_run_bulk_jobs, jids, jt, start, end, incr)
        jid = create_string_buffer(128)
        while drmaa_get_next_job_id(jids.contents, jid, 128)\
                != const.NO_MORE_ELEMENTS:
            yield jid.value
    finally:
        drmaa_release_job_ids(jids.contents)

def c(f, *args):
    return f(*(list(args) + [error_buffer, sizeof(error_buffer)]))

def string_vector(v):
    vlen = len(v)
    values = (STRING * (vlen + 1))()
    for i, el in enumerate(v):
        values[i] = STRING(el)
    values[vlen] = STRING()
    return values

def attribute_setter(obj, attribute_name):
    "returns a drmaa attribute setter"
    def f(value):
        "setter for %s" % attribute_name
        c(drmaa_set_attribute, obj, attribute_name, value)
    f.__name__ = 'set_'+attribute_name
    return f

def attribute_getter(obj, attribute_name):
    "returns a drmaa attribute setter"
    def f():
        "getter for %s" % attribute_name
        attr_buffer = create_string_buffer(1024)
        c(drmaa_get_attribute, obj, attribute_name, attr_buffer, sizeof(attr_buffer))
        return attr_buffer.value
    f.__name__ = 'get_'+attribute_name
    return f
